# Define a function which returns a value which fills in all the holes left by
# ``issuer.nix``.
{
  # A string giving the IP address and port number (":"-separated) of the VPN
  # server.
  monitoringvpnEndpoint

  # A string giving the domain name associated with this grid.  This is meant
  # to be combined with the hostname for this system to produce a
  # fully-qualified domain name.  For example, an issuer might have "payments"
  # as its hostname and belong to a grid with the domain
  # "example-grid.invalid".  This ``domain`` parameter should have the value
  # ``"example-grid.invalid"`` for the system figure out that
  # ``payments.example-grid.invalid`` is the name of this system.
, domain

  # A string giving an email address to use for Let's Encrypt registration and
  # certificate issuance.
, letsEncryptAdminEmail

  # A list of strings giving the domain names that point at this issuer
  # system.  These will all be included in Let's Encrypt certificate.
, issuerDomains

  # A list of strings giving CORS Origins will the issuer will be configured
  # to allow.
, allowedChargeOrigins
, ...
}:
{ config, ... }: 
let
  inherit (config.deployment) publicKeyPath privateKeyPath;
  inherit (config) monitoringvpnIPv4;
in {
  imports = [ ./deployment.nix ];
  # The morph default deployment target the name of the node in the network
  # attrset.  We don't always want to give the node its proper public address
  # there (because it depends on which domain is associated with the grid
  # being configured and using variable names complicates a lot of things).
  # Instead, just tell morph how to reach the node here - by using its fully
  # qualified domain name.
  deployment.targetHost = "${config.networking.hostName}.${config.networking.domain}";

  networking.domain = domain;

  services.private-storage.monitoring.vpn.client = {
    enable = true;
    ip = monitoringvpnIPv4;
    endpoint = monitoringvpnEndpoint;
    endpointPublicKeyFile = "${publicKeyPath}/monitoringvpn/server.pub";
  };

  services.private-storage-issuer = {
    inherit letsEncryptAdminEmail allowedChargeOrigins;
    domains = issuerDomains;
  };

  system.stateVersion = "19.03";
}
